export type ID = number | string | bigint;
export type With<T, K extends keyof T> = Required<Pick<T, K>>;
export type Without<E, P extends keyof E> = Omit<E, P>;
export type Component<E> = keyof E;
export type ComponentUpdater<T> = T | ((value: T) => T);
export type ComponentUpdaters<E, C extends keyof E> = [
    C,
    ComponentUpdater<E[C]>
];
export type ComponentUpdate<E, C extends keyof E> = [C, E[C]];
export interface BaseEntity {
    id: ID | undefined;
}
export type EntitiesCallback<Entity> = (entities: Set<Entity>) => any;
export type EntityCallback<Entity> = (entity: Entity) => any;
export type EntityUpdateCallback<Entity> = (entity: Entity, updates?: ComponentUpdate<Entity, Component<Entity>>[]) => any;
interface Query<Entity extends object, C extends keyof Entity> {
    components: C[];
    exclude?: C[];
    entities: Set<With<Entity, C>>;
    subscriptions: Set<EntitiesCallback<With<Entity, C>>>;
    updates: Set<EntityUpdateCallback<With<Entity, C>>>;
}
export interface Config<Entity> {
    getId: (e: Entity) => ID;
    parallel: boolean;
    asyncUpdates: boolean;
    generateId: () => ID;
    onBefore: (...args: any[]) => void | Promise<void>;
    onAfter: (...args: any[]) => void | Promise<void>;
}
export declare function getComponent<Entity extends {}, C extends keyof Entity>(entity: Entity, component: C): Entity[C];
export declare function hasComponent<Entity extends {} = any>(entity: Entity, component: Component<Entity>): boolean;
export declare function hasSomeComponents<Entity extends {} = any>(entity: Entity, components: (keyof Entity)[]): boolean;
export declare function hasComponents<Entity extends {} = any>(entity: Entity, components: (keyof Entity)[]): boolean;
export declare function generateId(): number;
export declare function makeQueryKey<Entity extends object, C extends Component<Entity>>(components: C[]): string;
export declare class World<Entity extends {} = any> {
    protected config: Config<Entity>;
    protected _entities: Map<ID, Entity>;
    protected systems: [Function, Set<Entity>][];
    protected queries: Map<string, Query<Entity, Component<Entity>>>;
    protected _onCreate: Set<EntityCallback<Entity>>;
    protected _onUpdate: Set<EntityUpdateCallback<Entity>>;
    protected _onRemove: Set<EntityCallback<Entity>>;
    get size(): number;
    get entities(): IterableIterator<Entity>;
    constructor(config?: Partial<Config<Entity>>);
    protected queryWithKey<C extends Component<Entity>>(key: string, components: C[], opts?: {
        exclude?: C[];
        persist?: Boolean;
    }): Set<With<Entity, C>>;
    get<C extends Component<Entity> = any>(id: ID): With<Entity, C> | undefined;
    add: (entity: Entity) => Entity;
    create(entity: Entity): Entity;
    remove(entity: Entity): Entity;
    clear(): void;
    query<C extends Component<Entity>>(components: C[], opts?: {
        exclude?: Component<Entity>[];
        persist?: Boolean;
    }): Set<With<Entity, C>>;
    register<C extends Component<Entity>>(system: Function, components: C[], exclude?: Component<Entity>[]): void;
    addComponents(entity: Entity, components: Partial<Entity>): void;
    removeComponents<C extends Component<Entity>>(entity: Entity, components?: C[]): void;
    run<Args extends readonly any[]>(...args: Args): Promise<void>;
    subscribe<C extends Component<Entity>>(components: C[], callback: EntitiesCallback<With<Entity, C>>, opts?: {
        emit?: boolean;
        exclude?: Component<Entity>[];
    }): () => void;
    makeSubscription<C extends Component<Entity>>(components: C[], opts?: {
        emit?: boolean;
        exclude?: Component<Entity>[];
    }): (cb: EntitiesCallback<With<Entity, C>>) => () => void;
    onEntityCreated(cb: EntityCallback<Entity>): () => void;
    onEntityUpdated(cb: EntityCallback<Entity>): () => void;
    onEntityRemoved(cb: EntityCallback<Entity>): () => void;
    onUpdate<C extends Component<Entity>>(components: C[], cb: EntityUpdateCallback<With<Entity, C>>): () => void;
    makeUpdateHandler<C extends Component<Entity>>(components: C[]): (cb: EntityUpdateCallback<With<Entity, C>>) => () => void;
    unsubscribe<C extends Component<Entity>>(components: C[], callback: EntitiesCallback<With<Entity, C>>): void;
    unsubscribeUpdate<C extends Component<Entity>>(components: C[], callback: EntityUpdateCallback<With<Entity, C>>): void;
    update(entity: Entity, components: Partial<Entity>): Entity;
    updateComponent<C extends Component<Entity>>(entity: Entity, component: C, update: ComponentUpdater<Entity[C]>): Entity[C];
    updateComponents<C extends keyof Entity>(entity: Entity, updates: ComponentUpdaters<Entity, C>[]): ComponentUpdate<Entity, C>[];
    private _update;
    private _handleUpdates;
    private _handleUpdateCallbacks;
    private _handleRemoveCallbacks;
}
export {};
